==> common-dto-4.2-sources.jar: fr/ird/observe/spi/map/ImmutableSetDtoMap.java <==
package fr.ird.observe.spi.map;

/*-
 * #%L
 * ObServe Toolkit :: Common Dto
 * %%
 * Copyright (C) 2017 - 2018 IRD, Ultreia.io
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-3.0.html>.
 * #L%
 */

import com.google.common.collect.ImmutableMap;
import fr.ird.observe.dto.IdHelper;
import fr.ird.observe.dto.reference.ReferentialDtoReference;
import java.util.Collection;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * A special map to store string values using the {@link #key0(Class)} method to produce keys.
 * <p>
 * Created by tchemit on 01/09/17.
 *
 * @author Tony Chemit - dev@tchemit.fr
 */
public class ImmutableSetDtoMap<V> {

    private static final transient Function<Class, String> keyFunction = IdHelper::getDtoSimplifiedName;
    private final ImmutableMap<String, Set<V>> data;
    private final ImmutableMap<String, Class> types;

    public static <V> Builder<V> builder() {
        return new Builder<>();
    }

    ImmutableSetDtoMap(ImmutableMap<String, Set<V>> data, ImmutableMap<String, Class> types) {
        this.data = data;
        this.types = types;
    }

    public int size() {
        return data.size();
    }

    public boolean isEmpty() {
        return data.isEmpty();
    }

    public Set<V> get(Class key) {
        return data.get(key0(key));
    }

    public Set<String> keySet() {
        return data.keySet();
    }

    public Collection<Set<V>> values() {
        return data.values();
    }

    public boolean containsEntry(Class type, V value) {
        String key = key0(type);
        return data.containsKey(key) && data.get(key).contains(value);
    }

    @SuppressWarnings("unchecked")
    public Set<Class<? extends ReferentialDtoReference>> referentialReferenceTypes() {
        return (Set) types.values().stream().filter(IdHelper::isReferential).collect(Collectors.toSet());
    }

    public static class Builder<V> {

        private final Map<String, Set<V>> data;
        private final ImmutableMap.Builder<String, Class> types;

        public Builder() {
            this.data = new TreeMap<>();
            this.types = ImmutableMap.builder();
        }

        public Builder put(Class type, V value) {
            String key = keyFunction.apply(type);
            data.getOrDefault(key, new LinkedHashSet<>()).add(value);
            types.put(key, type);
            return this;
        }

        @SuppressWarnings("unchecked")
        public <VV extends V> Builder put(Class type, Set<VV> value) {
            String key = keyFunction.apply(type);
            data.put(key, (Set<V>) value);
            types.put(key, type);
            return this;
        }

        public ImmutableSetDtoMap<V> build() {
            return new ImmutableSetDtoMap<>(ImmutableMap.copyOf(data), types.build());
        }

    }

    private String key0(Class type) {
        return keyFunction.apply(type);
    }
}

==> common-dto-4.13-sources.jar: fr/ird/observe/spi/map/ImmutableSetDtoMap.java <==
package fr.ird.observe.spi.map;

/*-
 * #%L
 * ObServe Toolkit :: Common Dto
 * %%
 * Copyright (C) 2017 - 2019 IRD, Ultreia.io
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-3.0.html>.
 * #L%
 */

import com.google.common.collect.ImmutableMap;

import java.util.Collection;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;

/**
 * A special map to store string values using the {@link #key0(Class)} method to produce keys.
 * <p>
 * Created by tchemit on 01/09/17.
 *
 * @author Tony Chemit - dev@tchemit.fr
 */
public class ImmutableSetDtoMap<V> extends ImmutableSetMap<V> {

    ImmutableSetDtoMap(ImmutableMap<String, Set<V>> data, ImmutableMap<String, Class> types) {
        super(data, types);
    }

    public static <V> Builder<V> builder() {
        return new Builder<>();
    }

    public static class Builder<V> {

        private final Map<String, Set<V>> data;
        private final ImmutableMap.Builder<String, Class> types;

        public Builder() {
            this.data = new TreeMap<>();
            this.types = ImmutableMap.builder();
        }

        public Builder put(Class type, V value) {
            String key = keyFunction.apply(type);
            data.computeIfAbsent(key, e -> new LinkedHashSet<>()).add(value);
            types.put(key, type);
            return this;
        }

        @SuppressWarnings("unchecked")
        public <VV extends V> Builder put(Class type, Collection<VV> value) {
            String key = keyFunction.apply(type);
            data.computeIfAbsent(key, e -> new LinkedHashSet<>()).addAll(value);
            types.put(key, type);
            return this;
        }

        public ImmutableSetDtoMap<V> build() {
            return new ImmutableSetDtoMap<>(ImmutableMap.copyOf(data), types.build());
        }

    }

}
